local util = include( "modules/util" )
local commondefs = include( "sim/unitdefs/commondefs" )
local simdefs = include( "sim/simdefs" )

local tool_templates =
{
    	augment_sombra = util.extend( commondefs.augment_template )
	{
		name = STRINGS.TALON.ITEMS.AUGMENTS.SOMBRAAUG,
		desc = STRINGS.TALON.ITEMS.AUGMENTS.SOMBRAAUG_TIP, 
		flavor = STRINGS.TALON.ITEMS.AUGMENTS.SOMBRAAUG_FLAVOR,
		onTooltip = function( tooltip, unit, userUnit )
		commondefs.onItemTooltip( tooltip, unit, userUnit )
			tooltip:addAbility( STRINGS.ITEMS.PROTOCHIP_HEADER, string.format( STRINGS.ITEMS.PROTOCHIP_FORMAT, unit:getTraits().ammo, unit:getTraits().maxAmmo ), "gui/items/icon-action_hack-console.png" )
			tooltip:addAbility( STRINGS.TALON.ITEMS.AUGMENTS.SOMBRAAUG_CAPACITOR, STRINGS.TALON.ITEMS.AUGMENTS.SOMBRAAUG_CAPACITOR_DESC, "gui/icons/arrow_small.png" )
			tooltip:addAbility(  STRINGS.TALON.ITEMS.NOTE_ACTION, STRINGS.TALON.ITEMS.NOTE_ACTION_DESC, "gui/icons/arrow_small.png" )	
		end,
		traits = util.extend( commondefs.DEFAULT_AUGMENT_TRAITS ){
			installed = true,
			addTrait = {{"sombra_wireless_range",8}, {"hasSombAug",true}},	
			ammo = 0,
			maxAmmo = 6,
			sombra_wireless_range = 8,
			addAbilities = "sombra_doorOpen",
		},	
		abilities = util.tconcat( commondefs.augment_template.abilities, { "sombra_jackin_charge", "sombra_icebreak" }),
		profile_icon = "gui/icons/Talon/icon-item_augment_sombra_small.png",
		profile_icon_100 = "gui/icons/Talon/icon-item_augment_sombra.png",		
		createUpgradeParams = function( self, unit )
			return { traits = { ammo = unit:getTraits().ammo } }
		end,
	},

	
	item_tazer_sombra = util.extend(commondefs.item_template)
	{
		name = STRINGS.TALON.ITEMS.SOMBRA_CLOAK,
		desc = STRINGS.TALON.ITEMS.SOMBRA_CLOAK_TOOLTIP,
		flavor = STRINGS.TALON.ITEMS.SOMBRA_CLOAK_FLAVOR,
		icon = "itemrigs/FloorProp_InvisiCloakTimed.png",
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_invisicloak_small.png",			
		profile_icon_100 = "gui/icons/item_icons/icon-item_invisi_cloak.png",
		traits = { disposable = false, duration = 1, cooldown = 0, cooldownMax = 10,  cloakDistanceMax=6, mpRestored=3, cloakInVision = false, sprintOnly = false, range = nil },
		requirements = { stealth = 2 },  
		abilities = { "carryable","recharge","sombra_useInvisiCloak" },
		value = 400,
		onTooltip = function( tooltip, unit, userUnit )
			commondefs.onItemTooltip( tooltip, unit, userUnit )
			tooltip:addAbility( STRINGS.TALON.ITEMS.SOMBRA_CLOAK_BONUSMP, STRINGS.TALON.ITEMS.SOMBRA_CLOAK_BONUSMP_DESC, "gui/icons/arrow_small.png" )
		end,
	},

	item_translocator = util.extend( commondefs.grenade_template )
	{
		name = STRINGS.TALON.ITEMS.SOMBRA_TRANSLOCATOR,
		desc = STRINGS.TALON.ITEMS.SOMBRA_TRANSLOCATOR_TOOLTIP,
		flavor = STRINGS.TALON.ITEMS.SOMBRA_TRANSLOCATOR_FLAVOR,
		icon = "itemrigs/FloorProp_Bandages.png",
		profile_icon = "gui/icons/Talon/icon-item_translocator_small.png",	
		profile_icon_100 = "gui/icons/Talon/icon-item_translocator.png",	
		kanim = "kanim_hologrenade",
		sounds = {activate="SpySociety/Actions/holocover_activate", deactivate="SpySociety/Actions/holocover_deactivate", activeSpot="SpySociety/Actions/holocover_run_LP", bounce="SpySociety/Grenades/bounce"},
		traits = { maxAmmo = 3, ammo = 1, disposable = false, agent_filter=true, beaconLifeMax = 1,
			restrictedUse={{agentID="SOMBRA_001",name=STRINGS.TALON.AGENTS.SOMBRA.NAME_LC}}},	
		abilities = { "restock", "carryable", "sombra_tport_translocator", "sombra_throw_translocator" },
		value = 600,		
		locator=true,
		createUpgradeParams = function( self, unit )
			return { traits = { ammo =  math.min(unit:getTraits().ammo + 1,  unit:getTraits().maxAmmo) } }
		end,
		onTooltip = function( tooltip, unit, userUnit )
			commondefs.onItemTooltip( tooltip, unit, userUnit )
			tooltip:addAbility( util.sformat(STRINGS.TALON.ABILITIES.RESTOCKABLE, unit:getTraits().ammo or 0, unit:getTraits().maxAmmo or 0), STRINGS.TALON.ABILITIES.RESTOCKABLE_DESC, "gui/icons/arrow_small.png" )
		end,
	},
	
	item_translocator_true = util.extend(commondefs.item_template)
	{
		type = "simemppack",
		name = "STRINGS.TALON.ITEMS.SOMBRA_TRANSLOCATOR",
		desc = STRINGS.TALON.ITEMS.WIDOW_MINE_TIP,
		profile_icon = "gui/icons/Talon/icon-item_venom_mine_small.png",
		profile_icon_100 = "gui/icons/Talon/icon-item_venom_mine.png",	
		kanim = "kanim_somb_translocator",	
		abilities = {},
		traits = { cooldown = 0, cooldownMax = 5, decoy_pack = true, flash_pack = true, sightable = true, hidesInCover = true, trigger_mainframe=false, range = 4, canSleep = false, baseDamage = 0, mainframe_icon_on_deploy=true, mainframe_status = "active", deployed = true, laptop = true},
		sounds = {activate="SpySociety/Actions/holocover_activate", deactivate="SpySociety/Actions/holocover_deactivate", activeSpot="SpySociety/Actions/holocover_run_LP", bounce="SpySociety/Grenades/bounce"},
	},

}

-- Reassign key name to value table.
for id, template in pairs(tool_templates) do
	template.id = id
end

return tool_templates


















