local util = include( "modules/util" )
local commondefs = include( "sim/unitdefs/commondefs" )
local simdefs = include( "sim/simdefs" )

local tool_templates =
{
	augment_widowmaker = util.extend( commondefs.augment_template )
	{
		name = STRINGS.TALON.ITEMS.AUGMENTS.WIDOWSIGHT, 
		desc = STRINGS.TALON.ITEMS.AUGMENTS.WIDOWSIGHT_TIP,  
		flavor = STRINGS.TALON.ITEMS.AUGMENTS.WIDOWSIGHT_FLAVOR, 
		traits = util.extend( commondefs.DEFAULT_AUGMENT_TRAITS ){		
			installed = true,
			isWidowAug = true,	
			addTrait = {{"widowsight_range",6}, {"hasWidowmakerAug",true}},
		},	
		abilities = util.tconcat( commondefs.augment_template.abilities ), 
		profile_icon = "gui/icons/Talon/icon-item_augment_widow_small.png",
		profile_icon_100 = "gui/icons/Talon/icon-item_augment_widow.png",		
	},
	
	item_widowsgun = util.extend( commondefs.weapon_template )
	{
		name = STRINGS.TALON.ITEMS.WIDOW_RIFLE,
		desc = STRINGS.TALON.ITEMS.WIDOW_RIFLE_TOOLTIP,
		flavor = STRINGS.TALON.ITEMS.WIDOW_RIFLE_FLAVOR,
		icon = "itemrigs/FloorProp_Rifle.png",	
		profile_icon = "gui/icons/Talon/icon-item_widowgun_small.png",	
		profile_icon_100 = "gui/icons/Talon/icon-item_widowgun.png",				
		equipped_icon = "gui/items/equipped_rifle.png",
		traits = { weaponType="rifle", baseDamage = 2, armorPiercing = 0, canSleep = true, charges = 2, chargesMax = 2, usesCharges = true,
			isWidowGun = true, isAutoMode = true, storedAmmo = 1, storedCharges = nil, ammo = nil, maxAmmo = nil,
			sniperDamage = 2, sniperArmorPiercing = 1, sniperMaxAmmo = 1, sniperMPCost = 4,
			autoDamage = 2, autoArmorPiercing = 0, autoChargesMax = 2,
			restrictedUse={{agentID="WIDOWMAKER_001",name=STRINGS.TALON.AGENTS.WIDOWMAKER.NAME_LC}} },
		abilities = util.tmerge( { "reload" }, { "recharge" }, { "widow_guntoggle" },  { "equippable" }, commondefs.weapon_template.abilities ),
		sounds = {shoot="SpySociety/Weapons/Precise/shoot_rifle_silenced", reload="SpySociety/Weapons/Precise/reload_rifle", use="SpySociety/Actions/item_pickup",shell="SpySociety/Weapons/Shells/shell_rifle_wood"},
		weapon_anim = "kanim_precise_smg",
		agent_anim = "anims_2h",
		createUpgradeParams = function( self, unit )
			return { traits = { storedAmmo = unit:getTraits().storedAmmo or unit:getTraits().ammo, charges =  unit:getTraits().chargesMax or unit:getTraits().autoChargesMax} }
		end,
		value = 1800,
		onTooltip = function( tooltip, unit, userUnit )
			commondefs.weapon_template.onTooltip( tooltip, unit, userUnit )

			local armorPiercing = unit:getTraits().armorPiercing or 0
			local damage = unit:getTraits().baseDamage or 0
			local curAuto = unit:getTraits().isAutoMode or 0
			local assultStoredCharges = unit:getTraits().storedCharges or 0
			local assultChargesMax = unit:getTraits().autoChargesMax or 0
			local storedAmmo = unit:getTraits().storedAmmo or 0
			local curAmmo = unit:getTraits().ammo or 0
			local sniperMaxAmmo = unit:getTraits().sniperMaxAmmo or 0
			local sniperMod = unit:getTraits().sniperArmorPiercing or 0

			if curAuto == true then
				tooltip:addAbility( STRINGS.TALON.ITEMS.WIDOW_RIFLE_TOOLTIP_TOGGLABLE, util.sformat(STRINGS.TALON.ITEMS.WIDOW_RIFLE_TOOLTIP_TOGGLE_S_DESC, armorPiercing+ sniperMod , storedAmmo, sniperMaxAmmo), "gui/icons/arrow_small.png" )
			else
				tooltip:addAbility( STRINGS.TALON.ITEMS.WIDOW_RIFLE_TOOLTIP_TOGGLABLE, util.sformat(STRINGS.TALON.ITEMS.WIDOW_RIFLE_TOOLTIP_TOGGLE_A_DESC, damage, armorPiercing - sniperMod, assultStoredCharges, assultChargesMax), "gui/icons/arrow_small.png" )
			end
		end,
	},
	
	item_venommine = util.extend( commondefs.grenade_template )
	{
		name = STRINGS.TALON.ITEMS.WIDOW_MINE,
		desc = STRINGS.TALON.ITEMS.WIDOW_MINE_TOOLTIP,
		flavor = STRINGS.TALON.ITEMS.WIDOW_MINE_FLAVOR,
		profile_icon = "gui/icons/Talon/icon-item_venom_mine_small.png",	
		profile_icon_100 = "gui/icons/Talon/icon-item_venom_mine.png",	
		kanim = "kanim_stickycam",
		sounds = {activate="SpySociety/Actions/holocover_activate", deactivate="SpySociety/Actions/holocover_deactivate", activeSpot="SpySociety/Actions/holocover_run_LP", bounce="SpySociety/Grenades/bounce"},
		traits = { disposable = false, agent_filter=true, cooldown = 0, cooldownMax = 7,},	
		abilities = { "recharge", "carryable", "widow_throw_mine" },
		value = 600,		
		locator=true,
	},
	
	item_venommine_true = util.extend(commondefs.item_template)
	{
		name = STRINGS.TALON.ITEMS.WIDOW_MINE,
		desc = STRINGS.TALON.ITEMS.WIDOW_MINE_TIP,
		profile_icon = "gui/icons/Talon/icon-item_venom_mine_small.png",
		profile_icon_100 = "gui/icons/Talon/icon-item_venom_mine.png",	
		kanim = "kanim_venommine",
		abilities = {},
		traits = { cooldown = 0, cooldownMax = 7, mainframe_status = "active" },
		sounds = {activate="SpySociety/Actions/holocover_activate", deactivate="SpySociety/Actions/holocover_deactivate", activeSpot="SpySociety/Actions/holocover_run_LP", bounce="SpySociety/Grenades/bounce"},
	},
}

	
	

-- Reassign key name to value table.
for id, template in pairs(tool_templates) do
	template.id = id
end

return tool_templates