---------------------------------------------------------------------
-- Invisible Inc. Mod.
--
local util = include( "modules/util" )
local commondefs = include( "sim/unitdefs/commondefs" )
local simdefs = include( "sim/simdefs" )

local tool_templates =
{
	
		item_widowsgun = util.extend( commondefs.weapon_template )   ---  Sniper uses single ammo, assault uses 2 charges.
	{
		name = STRINGS.TALON.ITEMS.WIDOW_RIFLE,
		desc = STRINGS.TALON.ITEMS.WIDOW_RIFLE_TOOLTIP,
		flavor = STRINGS.TALON.ITEMS.WIDOW_RIFLE_FLAVOR,
		icon = "itemrigs/FloorProp_Rifle.png",	
		--profile_icon = "gui/items/icon-item_gun_pistol.png",
		profile_icon = "gui/icons/Talon/icon-item_widowgun_small.png",	
		profile_icon_100 = "gui/icons/Talon/icon-item_widowgun.png",				
		equipped_icon = "gui/items/equipped_rifle.png",
		traits = { weaponType="rifle", baseDamage = 2, armorPiercing = 0, canSleep = true, charges = 2, chargesMax = 2, usesCharges = true,
											isWidowGun = true, isAutoMode = true, storedAmmo = 1, storedCharges = nil, ammo = nil, maxAmmo = nil,
											--ammo_clip = 1,
											sniperDamage = 2, sniperArmorPiercing = 1, sniperMaxAmmo = 1, sniperMPCost = 4,
											autoDamage = 2, autoArmorPiercing = 0, autoChargesMax = 2,
												restrictedUse={{agentID="WIDOWMAKER_001",name=STRINGS.TALON.AGENTS.WIDOWMAKER.NAME_LC}} },
        abilities = util.tmerge( { "recharge" }, { "widow_reload" }, { "widow_guntoggle" }, commondefs.weapon_template.abilities ),
		sounds = {shoot="SpySociety/Weapons/Precise/shoot_rifle_silenced", reload="SpySociety/Weapons/Precise/reload_rifle", use="SpySociety/Actions/item_pickup",shell="SpySociety/Weapons/Shells/shell_rifle_wood"},
		weapon_anim = "kanim_precise_smg",
		agent_anim = "anims_2h",
		createUpgradeParams = function( self, unit )
			return { traits = { storedAmmo = unit:getTraits().storedAmmo or unit:getTraits().ammo, charges = math.min((unit:getTraits().charges or unit:getTraits().storedCharges) + 1, 2)} }
		end,
		value = 1800,
		--soldAfter = 24, 
		--floorWeight = 3,
		onTooltip = function( tooltip, unit, userUnit )
			--commondefs.onItemTooltip( tooltip, unit, userUnit )
			local HOTKEY_COLOUR = "<c:ffffff>"
			local FLAVOUR_COLOUR = "<c:61AAAA>"
			local ITEM_HEADER_COLOUR = "<ttheader>"
			local DESC_COLOUR = "<c:ffffff>"
			local EQUIPPED_COLOUR = "<c:FF8411>"
			local NOTINSTALLED_COLOUR = "<c:FF8411>"
			--local SPECIAL_INFO_COLOUR = "<c:F4FF78>"
			local SPECIAL_INFO_COLOUR = "<c:ffffff>"
			local simquery = include( "sim/simquery" )
			local name = util.toupper( unit:getName() )
			if unit:getTraits().ammo and unit:getTraits().maxAmmo then
	--			tooltip:addLine( ITEM_HEADER_COLOUR..name.."</>", string.format(STRINGS.ITEMS.AMMO,unit:getTraits().ammo,unit:getTraits().maxAmmo) )
			else
				tooltip:addLine( ITEM_HEADER_COLOUR..name.."</>" )
			end	
			
			if unit:getUnitData().flavor then
				tooltip:addDesc(  FLAVOUR_COLOUR..unit:getUnitData().flavor.."</>" )
			end				
			
			if unit:getUnitData().desc then
				tooltip:addDesc( unit:getUnitData().desc )
			end
			
			if unit:getTraits().equipped then
				tooltip:addLine( EQUIPPED_COLOUR.. STRINGS.UI.TOOLTIPS.EQUIPPED .."</c>") 
			end
			
			if unit:getTraits().restrictedUse then
				local  agentList = ""
				local skip = true
				for i,agent in ipairs(unit:getTraits().restrictedUse) do	
					if skip == false then
						agentList = agentList .."\n"
					end		
					skip = false
					agentList = agentList .. agent.name
				end
				tooltip:addAbility( STRINGS.ITEMS.TOOLTIPS.RESTRICTED_USE, util.sformat(STRINGS.ITEMS.TOOLTIPS.RESTRICTED_USE_DESC, agentList), "gui/icons/arrow_small.png" )
			end	
			
			if not unit:getTraits().slot and unit:getTraits().damage then
				tooltip:addAbility( STRINGS.ITEMS.TOOLTIPS.KO_DAMAGE, util.sformat(STRINGS.ITEMS.TOOLTIPS.KO_DAMAGE_DESC, unit:getTraits().damage), "gui/icons/arrow_small.png" )
			end	
			
			if unit:getTraits().usesCharges then
				tooltip:addAbility( util.sformat(STRINGS.ITEMS.TOOLTIPS.USE_CHARGES, unit:getTraits().charges,unit:getTraits().chargesMax), STRINGS.TALON.ITEMS.WIDOW_RIFLE_CHARGES_DESC_FREE, "gui/icons/arrow_small.png" )
			end	
			
			
			------------------------------
			local sim = unit._sim 
			local armorPiercing = unit:getTraits().armorPiercing or 0
			local damage = unit:getTraits().baseDamage or 0
			local curAuto = unit:getTraits().isAutoMode or 0
			local assultStoredCharges = unit:getTraits().storedCharges or 0
			local assultChargesMax = unit:getTraits().autoChargesMax or 0
			local storedAmmo = unit:getTraits().storedAmmo or 0
			local curAmmo = unit:getTraits().ammo or 0
			local sniperMaxAmmo = unit:getTraits().sniperMaxAmmo or 0
			local sniperMod = unit:getTraits().sniperArmorPiercing or 0

			if unit:getTraits().ignoreArmor then
				tooltip:addAbility( STRINGS.ITEMS.TOOLTIPS.IGNORES_ARMOR, STRINGS.ITEMS.TOOLTIPS.IGNORES_ARMOR_DESC, "gui/icons/arrow_small.png" )
			else
				if armorPiercing > 0 then
					tooltip:addAbility( STRINGS.ITEMS.TOOLTIPS.ARMOR_PIERCING, string.format(STRINGS.ITEMS.TOOLTIPS.ARMOR_PIERCING_DESC, armorPiercing), "gui/icons/arrow_small.png" )
				else			
					tooltip:addAbility( STRINGS.ITEMS.TOOLTIPS.ARMOR_PIERCING_NONE, STRINGS.ITEMS.TOOLTIPS.ARMOR_PIERCING_NONE_DESC, "gui/icons/arrow_small.png" )
				end
			end

			if damage > 0 and unit:getTraits().canSleep then
				tooltip:addAbility(  STRINGS.ITEMS.TOOLTIPS.KO_DAMAGE, util.sformat(STRINGS.ITEMS.TOOLTIPS.KO_DAMAGE_DESC, damage), "gui/icons/arrow_small.png" )
			end

			if unit:getTraits().canTag then
				tooltip:addAbility( STRINGS.ITEMS.TOOLTIPS.CANTAG, string.format(STRINGS.ITEMS.TOOLTIPS.CANTAG_DESC), "gui/icons/arrow_small.png" )
			end
			
			if unit:getTraits().ammo then
				tooltip:addAbility( util.sformat(STRINGS.TALON.ITEMS.WIDOW_RIFLE_AMMO, curAmmo, sniperMaxAmmo), STRINGS.TALON.ITEMS.WIDOW_RIFLE_AMMO_DESC, "gui/icons/arrow_small.png" )
			end

			if curAuto == true then
				tooltip:addAbility( STRINGS.TALON.ITEMS.WIDOW_RIFLE_TOOLTIP_TOGGLABLE, util.sformat(STRINGS.TALON.ITEMS.WIDOW_RIFLE_TOOLTIP_TOGGLE_S_DESC, armorPiercing+ sniperMod , storedAmmo, sniperMaxAmmo), "gui/icons/arrow_small.png" )
			else
				tooltip:addAbility( STRINGS.TALON.ITEMS.WIDOW_RIFLE_TOOLTIP_TOGGLABLE, util.sformat(STRINGS.TALON.ITEMS.WIDOW_RIFLE_TOOLTIP_TOGGLE_A_DESC, damage, armorPiercing - sniperMod, assultStoredCharges, assultChargesMax), "gui/icons/arrow_small.png" )
			end
		end,
	},
	


}

	
	

-- Reassign key name to value table.
for id, template in pairs(tool_templates) do
	template.id = id
end

return tool_templates