local util = include( "modules/util" )

local function earlyInit( modApi )
	modApi.requirements = { "Contingency Plan", "Function Library" }
end

local function init( modApi )
	local dataPath = modApi:getDataPath()
	local scriptPath = modApi:getScriptPath()
    
    	KLEIResourceMgr.MountPackage( dataPath .. "/gui.kwad", "data" )
	KLEIResourceMgr.MountPackage( dataPath .. "/anims.kwad", "data" )
    
	modApi:addGenerationOption("sombraload", STRINGS.TALON.OPTIONS.ENABLE_SOMBRA , STRINGS.TALON.OPTIONS.ENABLE_SOMBRA_TIP)
	modApi:addGenerationOption("widowload", STRINGS.TALON.OPTIONS.ENABLE_WIDOW , STRINGS.TALON.OPTIONS.ENABLE_WIDOW_TIP)
	modApi:addGenerationOption("noverlay", STRINGS.TALON.OPTIONS.DISABLE_OVERLAYS , STRINGS.TALON.OPTIONS.DISABLE_OVERLAYS_TIP)
	modApi:addGenerationOption("rescueagents", STRINGS.TALON.OPTIONS.DETCENTER , STRINGS.TALON.OPTIONS.DETCENTER_TIP)

	modApi:addAbilityDef( "sombra_hack", scriptPath .."/sombra/sombra_hack" ) 
	modApi:addAbilityDef( "sombra_jackin_charge", scriptPath .."/sombra/sombra_jackin_charge" ) 
	modApi:addAbilityDef( "sombra_icebreak", scriptPath .."/sombra/sombra_icebreak" ) 
	modApi:addAbilityDef( "sombra_wireless_scan", scriptPath .."/sombra/sombra_wireless_scan" ) 
	modApi:addAbilityDef( "sombra_useInvisiCloak", scriptPath .."/sombra/sombra_useInvisiCloak" ) 
	modApi:addAbilityDef( "sombra_throw_translocator", scriptPath .."/sombra/sombra_throw_translocator" ) 
	modApi:addAbilityDef( "sombra_tport_translocator", scriptPath .."/sombra/sombra_tport_translocator" ) 
	modApi:addAbilityDef( "sombra_doorOpen", scriptPath .."/sombra/sombra_doorOpen" ) 
	
	modApi:addAbilityDef( "widow_guntoggle", scriptPath .."/widowmaker/widow_guntoggle" )
	modApi:addAbilityDef( "widow_observePath", scriptPath .."/widowmaker/widow_observePath" )
	modApi:addAbilityDef( "widow_throw_mine", scriptPath .."/widowmaker/widow_throw_mine" )
	modApi:addAbilityDef( "widow_rappel", scriptPath .."/widowmaker/widow_rappel" )
	modApi:addAbilityDef( "widow_reload", scriptPath .."/widowmaker/widow_reload" )

	modApi:addAbilityDef( "restock", scriptPath .."/shared/restock" ) 
	modApi:addAbilityDef( "peek", scriptPath .."/shared/peek" ) 
	include(scriptPath .."/shared/simquery")
	include(scriptPath .."/widowmaker/smoke_cloud_poison")
end

local function load( modApi, options )
	local scriptPath = modApi:getScriptPath()

	local propdefs = include( scriptPath .. "/widowmaker/propdefs_smallSmoke" )
	for name, propdef in pairs(propdefs) do
		modApi:addPropDef( name, propdef, false )
	end
	
	if options["noverlay"].enabled then
        local animdefs = include( scriptPath .. "/animdefs_noverlay" )
		for name, animDef in pairs(animdefs) do
			modApi:addAnimDef( name, animDef )
		end   
	else
		local animdefs = include( scriptPath .. "/animdefs" )
		for name, animDef in pairs(animdefs) do
			modApi:addAnimDef( name, animDef )
		end  
	end
  
	if options["sombraload"].enabled then
        	local agentdefs = include( scriptPath .. "/agentdefs_sombra" )
        	for name, agentDef in pairs(agentdefs) do
			modApi:addAgentDef( name, agentDef, { name } )
		end
        
		local itemdefs = include( scriptPath .. "/itemdefs_sombra" )
		for name, itemDef in pairs(itemdefs) do
			modApi:addItemDef( name, itemDef )
		end  
		if options["rescueagents"] and options["rescueagents"].enabled then
			local serverdefs = include( "modules/serverdefs" )
			modApi:addRescueAgent( serverdefs.createAgent( "sombra_1", {"augment_sombra"} ) )
		end		
	end
	
	if options["widowload"].enabled then
        	local agentdefs = include( scriptPath .. "/agentdefs_widow" )
		for name, agentDef in pairs(agentdefs) do
			modApi:addAgentDef( name, agentDef, { name } )
		end
        
        	local itemdefs = include( scriptPath .. "/itemdefs_widow" )
		for name, itemDef in pairs(itemdefs) do
            		modApi:addItemDef( name, itemDef )
        	end    
		if options["rescueagents"] and options["rescueagents"].enabled then
			local serverdefs = include( "modules/serverdefs" )
			modApi:addRescueAgent( serverdefs.createAgent( "widowmaker_1", {"augment_widowmaker"} ) )
		end		
	end
end

local function initStrings( modApi )
	local dataPath = modApi:getDataPath()
	local scriptPath = modApi:getScriptPath()
	local MOD_STRINGS = include( scriptPath .. "/strings" )
	modApi:addStrings( dataPath, "TALON", MOD_STRINGS )  
end
 
return {
	earlyInit = earlyInit,
	init = init,
	load = load,
	initStrings = initStrings,
}
