local array = include( "modules/array" )
local util = include( "modules/util" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local abilityutil = include( "sim/abilities/abilityutil" )
local unitdefs = include( "sim/unitdefs" )
local mathutil = include( "modules/mathutil" )
local abilitydefs = include( "sim/abilitydefs" )

-------------------------------------------------------------------

local oldPeek = abilitydefs.lookupAbility("peek")

local oldDoPeek = oldPeek.doPeek

local peek = util.extend(oldPeek) {
	usesMP = true,

	getProfileIcon = function( self, sim, abilityOwner, abilityUser, hasTarget )	
		if abilityUser and abilityUser:getTraits().hasWidowmakerAug then	
			return "gui/icons/action_icons/Action_icon_Small/icon-item_peek_small.png"
		else
			if hasTarget then
	    			return "gui/icons/action_icons/Action_icon_Small/icon-action_peek_door.png"
	    		else
	        		return "gui/icons/action_icons/Action_icon_Small/icon-action_peek_around.png"
	    		end
		end
	end,

	doPeek = function( self, unit, view360, sim, x0,y0, peekInfo, dx, dy, exit )
		if unit:getTraits().hasWidowmakerAug then
			self:performScan( sim, unit )
		end
		oldDoPeek( self, unit, view360, sim, x0,y0, peekInfo, dx, dy, exit )
	end,
		
	performScan = function( self, sim, abilityOwner )
        	local player = abilityOwner:getPlayerOwner()
		local cell = sim:getCell( abilityOwner:getLocation() )
		local maxRange = abilityOwner:getTraits().widowsight_range
		local newGuards = {}
		
		for _, guardUnit in pairs(sim:getNPC():getUnits()) do
			local x1, y1 = guardUnit:getLocation()
			if x1 and y1 and guardUnit:getTraits().isGuard then
				local distance = mathutil.dist2d( cell.x, cell.y, x1, y1 )
				if distance < maxRange then   	
					player:glimpseUnit( sim, guardUnit:getID() )
					sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = guardUnit } )  
					if not sim:canPlayerSeeUnit( player, guardUnit ) and not guardUnit:getTraits().widowTagged_atLeastOnce then
						table.insert( newGuards, guardUnit )
					end
					guardUnit:getTraits().widowTagged = true
				end
			end
		end
			
		if #newGuards > 0 then
			local newGuard = newGuards[ sim:nextRand( 1, #newGuards ) ]	            
			sim:dispatchEvent( simdefs.EV_CAM_PAN, { newGuard:getLocation() } )	
			sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/mainframe_object_camera" )
			newGuard:getTraits().widowTagged_atLeastOnce = true
		end	
	end,
}

return peek