local array = include( "modules/array" )
local util = include( "modules/util" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local abilityutil = include( "sim/abilities/abilityutil" )
local inventory = include("sim/inventory")


local restock =
{
	name = STRINGS.TALON.ABILITIES.RESTOCK,
	createToolTip = function( self,sim,unit,targetCell)
		return abilityutil.formatToolTip(STRINGS.TALON.ABILITIES.RESTOCK, STRINGS.ABILITIES.RELOAD_DESC, 1 )
	end,

	profile_icon = "gui/icons/Talon/icon-item_restock_small.png",

	alwaysShow = true,

	getName = function( self, sim, unit )
		return STRINGS.TALON.ABILITIES.RESTOCK
	end,

	canUseAbility = function( self, sim, weaponUnit, unit )
		if weaponUnit:getTraits().ammo >= weaponUnit:getTraits().maxAmmo then
			return false, STRINGS.UI.REASON.AMMO_FULL
		end

		if weaponUnit:getTraits().noReload then 
			return false, STRINGS.UI.REASON.CANT_RELOAD
		end 

		local hasAmmo = array.findIf( unit:getChildren(), function( u ) 
			if u:getTraits().ammo_clip ~= nil and ( not u:getTraits().charges or (u:getTraits().charges and u:getTraits().charges > 0)) then
				return u:getTraits().ammo_clip
			else
				return nil
			end
		end )

		if not hasAmmo then 
			return false, STRINGS.UI.REASON.NO_MAGAZINE
		end

		return true
	end,
		
	executeAbility = function( self, sim, weaponUnit, unit )
		local userUnit = weaponUnit:getUnitOwner()

		local ammoConsumed = false 
		if weaponUnit:getTraits().infiniteAmmo then
			ammoConsumed = true
		elseif weaponUnit:getTraits().noReload then 
			ammoConsumed = false 
		elseif userUnit then
			local ammoClip = array.findIf( userUnit:getChildren(), function( u ) 
				if u:getTraits().ammo_clip ~= nil and ( not u:getTraits().charges or (u:getTraits().charges and u:getTraits().charges > 0)) then
					return u:getTraits().ammo_clip
				else 
					return nil 
				end 
			end )

			if ammoClip then
				ammoClip:getTraits().ammo_clip = ammoClip:getTraits().ammo_clip - 1
				if ammoClip:getTraits().ammo then
					ammoClip:getTraits().ammo = ammoClip:getTraits().ammo - 1
				end
				if ammoClip:getTraits().charges then
					ammoClip:getTraits().charges = ammoClip:getTraits().charges - 1
					ammoClip:getTraits().ammo_clip = ammoClip:getTraits().ammo_clip + 1
				end
				if ammoClip:getTraits().ammo_clip <= 0 or (ammoClip:getTraits().ammo and ammoClip:getTraits().ammo <= 0) then
					inventory.trashItem( sim, userUnit, ammoClip )
				end
				ammoConsumed = true
			end
		end

		if ammoConsumed then
			weaponUnit:getTraits().ammo = weaponUnit:getTraits().ammo + 1
			if userUnit then
				local x0, y0 = userUnit:getLocation()		    
				sim:emitSound( { path = weaponUnit:getUnitData().sounds.reload, range = simdefs.SOUND_RANGE_0 }, x0, y0, userUnit )
				if weaponUnit == simquery.getEquippedGun( userUnit ) then
					sim:dispatchEvent( simdefs.EV_UNIT_RELOADED, { unit = userUnit } )
				end
			end
		end

		return ammoConsumed
	end
}
return restock