local array = include( "modules/array" )
local util = include( "modules/util" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local inventory = include("sim/inventory")
local abilityutil = include( "sim/abilities/abilityutil" )

function getRoomMoveCost( cell1, cell2)
	-- if there is a door, this increases the cost.
	-- NOTE: since cell1 or cell2 may be ghosted cells, do NOT use a table compare to see if exit.cell == cell2!
	for k,exit in pairs(cell1.exits) do
		if exit.cell.x == cell2.x and exit.cell.y == cell2.y then
			return 1
		end
	end

	return 9 -- this effectively caps dist at 20 while still scanning door cells
end

function closeAllLockedDoors()
	self:forEachCell(
		function( c )
			for i,exit in pairs(c.exits) do
				if exit.locked and exit.keybits == simdefs.DOOR_KEYS.SECURITY then
					self:modifyExit( c, i, simdefs.EXITOP_UNLOCK)
                    self:getPC():glimpseExit( c.x, c.y, i )
				end
			end
		end )
end

function getAugment(sim, sourceUnit)
			if sourceUnit._children then
				for i,child in ipairs(sourceUnit._children) do
					-- fix for augments in inventory giving pirce boost		
						if child:getTraits()[ "sombra_wireless_range" ] then
							return child
						end
				end
			end
			return false
		end
		



local sombra_doorOpen = 
	{
		createToolTip = function( self,sim,unit,targetCell,dir)
			return abilityutil.formatToolTip(STRINGS.UI.HOTKEYS.USE_OPEN_DOOR, "Opens this door without unlocking it - the door closes again at the end of the player's turn.\nCosts 1 stored PWR", simdefs.DEFAULT_COST )
		end,

		profile_icon = "gui/icons/action_icons/Action_icon_Small/icon-action_door_open_small.png",
		
		eyeballs = {},
		
		getProfileIcon = function( self, sim, abilityOwner )
	        	return abilityOwner:getUnitData().profile_icon or self.profile_icon
	    	end,

        	proxy = true,

		getName = function( self, sim, unit )
			return string.format(STRINGS.ABILITIES.SET_NAME,unit:getName())
		end,
		
		onSpawnAbility = function( self, sim, unit )
			self.abilityOwner = unit
			
			local abilityOwner = self.abilityOwner
			sim:addTrigger( simdefs.TRG_END_TURN, self )
			sim:addTrigger( "usedStim", self )
			sim:addTrigger( simdefs.TRG_UNIT_WARP, self )
		end,
			
		onDespawnAbility = function( self, sim, unit )
			sim:removeTrigger( simdefs.TRG_END_TURN, self )
			sim:removeTrigger( "usedStim", self )
			sim:removeTrigger( simdefs.TRG_UNIT_WARP, self )
		end,
		
		
        
		acquireTargets = function( self, targets, game, sim, abilityOwner, abilityUser )
			local exits = {}
			local fromCell = sim:getCell( abilityUser:getLocation() )
			local cellsWithDoors = {}
			local player = abilityOwner:getPlayerOwner()
			local hasPower = true
		--	if (abilityOwner:getTraits().ammo or 0) <= 1) then
		--		hasPower = false
		--	end
			local cell = sim:getCell( abilityOwner:getLocation() )
			

			------todo- if abilityOwner next to gettraits mainframe_console or open_secure_boxes or generateAugment then
			if hasPower then
			--	local cells = simquery.floodFill( sim, nil, fromCell, 20, getRoomMoveCost, simquery.canPathBetween )  --- will find everything in same room
				local cells = simquery.fillCircle( sim, fromCell.x, fromCell.y, 8, 0)
				-------- floodFill( unit, start_cell, range, costFn, pathFn , maxRangeOnly, cellquery )
					for i, xc, yc in util.xypairs( cells ) do
						local cell = sim:getCell( xc, yc )
						if cell then
							local raycastX, raycastY = sim:getLOS():raycast(fromCell.x, fromCell.y, cell.x, cell.y)
							if raycastX == cell.x and raycastY == cell.y then
								

								for dir, exit in pairs( cell.exits ) do
									local rdir = simquery.getReverseDirection( dir )
									local reverseExit = exit.cell.exits[ rdir ]
									local x1, y1 = simquery.getDeltaFromDirection(dir)
									local toCell = sim:getCell( cell.x + x1, cell.y + y1 )
									if sim:canPlayerSee( player, cell.x, cell.y ) or sim:canPlayerSee( player, toCell.x, toCell.y ) then
								
										if simquery.isDoorExit( exit ) and not exit.consoleScanned and exit.keybits == simdefs.DOOR_KEYS.SECURITY
											and not (exit.keybits == simdefs.DOOR_KEYS.ELEVATOR or exit.keybits == simdefs.DOOR_KEYS.ELEVATOR_INUSE or exit.keybits == simdefs.DOOR_KEYS.GUARD) then
											table.insert( exits, { x = cell.x, y = cell.y, dir = dir } )
											exit.consoleScanned = true
											reverseExit.consoleScanned = true
										end
									end
								end
							end
						end
					end
				
				sim:forEachCell(
					function( c )
						for i,exit in pairs(c.exits) do
								exit.consoleScanned = nil
						end
					end )
			end
			return targets.exitTarget( game, exits, self, abilityOwner, abilityUser )
		end,

		canUseAbility = function( self, sim, abilityOwner, unit, cell )
            -- This is a proxy ability, but only usable if the proxy is in the inventory of the user.
			
			local augment = getAugment(sim, abilityOwner)
			
            if abilityOwner ~= unit and abilityOwner:getUnitOwner() ~= unit then
                return false
            end

			if abilityOwner:getTraits().cooldown and abilityOwner:getTraits().cooldown > 0 then
				return false, util.sformat(STRINGS.UI.REASON.COOLDOWN,abilityOwner:getTraits().cooldown)
			end	
			
			if unit:getMP() < 1 then
		--		return false, string.format(STRINGS.UI.REASON.REQUIRES_AP,1)
			end

			if abilityOwner:getTraits().usesCharges and abilityOwner:getTraits().charges < 1 then
				return false, util.sformat(STRINGS.UI.REASON.CHARGES)
			end	
			
			local exit = cell --cell.exits[dir]
					
			if unit:getMP() < 1 then
		--		return false, string.format(STRINGS.UI.REASON.REQUIRES_AP,1)
			end
			if (augment:getTraits().ammo or 0) < 1 then
				return false, "Requires 1 stored PWR"
			end		
		

			return abilityutil.checkRequirements( abilityOwner, unit )
		end,
		
		executeAbility = function( self, sim, unit, userUnit, x0, y0, dir )
			local fromCell = sim:getCell( x0, y0 )
            assert( fromCell and dir )
			local exit = fromCell.exits[dir]
			assert( exit )
			
			local augment = getAugment(sim, userUnit)
			
			userUnit:resetAllAiming()
			--sim:dispatchEvent( simdefs.EV_UNIT_USEDOOR, { unitID = unit:getID(), facing = newFacing } )		
		--	sim:dispatchEvent( simdefs.EV_UNIT_USECOMP, { unitID = unit:getID(), facing = newFacing, sound = "SpySociety/Actions/monst3r_jackin" , soundFrame = 16, useTinkerMonst3r=true } )
			local x1,y1 = userUnit:getLocation()
			local facing = simquery.getDirectionFromDelta( x0 - x1, y0 - y1 )
			simquery.suggestAgentFacing(unit, facing)
		--sim:dispatchEvent( simdefs.EV_UNIT_WIRELESS_SCAN, { unitID = unit:getID(), targetUnitID = target, hijack = true } )

			if not exit.locked then
			--	
				sim:modifyExit(fromCell, dir, simdefs.EXITOP_TOGGLE_DOOR, unit, false)	
				unit:useMP(1, sim)
			elseif exit.keybits == simdefs.DOOR_KEYS.SECURITY then			
			--		
				sim:modifyExit(fromCell, dir, simdefs.EXITOP_TOGGLE_DOOR, unit, false)	
				exit.closeEndTurn = true
			--	unit:useMP(1, sim)
				augment:getTraits().ammo = math.max( 0, augment:getTraits().ammo - 1 )
			elseif exit.keybits == simdefs.DOOR_KEYS.VAULT then		
			-- 
				local delay = 0.65
				sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60*delay)			
				sim:modifyExit(fromCell, dir, simdefs.EXITOP_TOGGLE_DOOR, unit, false)	
				exit.closeEndTurn = true
				unit:getTraits().mp = 0
			end
			--- find nearby human guards
			--- add tolerance counter
			--- break door if too intolerant
				--- no need now due to EMP!
		
			sim:processReactions( userUnit )
			--sim:dispatchEvent( simdefs.EV_UNIT_USEDOOR_PST, { unitID = unit:getID(), facing = newFacing } )	
		end,
	
		onTrigger = function ( self, sim, evType, evData )  -- need to get this working

			if evType == simdefs.TRG_END_TURN then
				local abilityOwner = self.abilityOwner
				local player = abilityOwner:getPlayerOwner()

				if player and sim:getCurrentPlayer() == player then				
					sim:forEachCell(
						function( c )
							for i,exit in pairs(c.exits) do
								if exit.locked and not exit.closed then
								--	sim:modifyExit( c, i, simdefs.EXITOP_CLOSE)
									--self:getPC():glimpseExit( c.x, c.y, i )
								end
							end
						end )					
				end
			end	

		end,


	}
return sombra_doorOpen