local array = include( "modules/array" )
local util = include( "modules/util" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local inventory = include("sim/inventory")
local abilityutil = include( "sim/abilities/abilityutil" )
local mathutil = include( "modules/mathutil" )

function icewall(icebreak)
	if icebreak then
		return math.max(1, math.floor((icebreak/2)+0.5))
	end
end


local sombra_icebreak =
{
	name = STRINGS.TALON.ITEMS.AUGMENTS.SOMBRAAUG_ICEB,
	profile_icon = "gui/icons/Talon/icon-item_sombra_hijack_small.png", 
    proxy = true,
	usesAction = true,

	createToolTip = function( self,sim, abilityOwner, abilityUser, targetID )
		local targetUnit = sim:getUnit( targetID )
		local headerTxt = util.sformat( STRINGS.TALON.ITEMS.AUGMENTS.SOMBRAAUG_TARGNAME, util.toupper( targetUnit:getName() ) )
		local bodyTxt = util.sformat( STRINGS.TALON.ITEMS.AUGMENTS.SOMBRAAUG_WILLDO, icewall(targetUnit:getTraits().mainframe_ice), abilityOwner:getTraits().ammo )
		return abilityutil.formatToolTip( headerTxt, bodyTxt, simdefs.DEFAULT_COST )
	end,
		
	getName = function( self, sim, unit )
		return STRINGS.TALON.ITEMS.AUGMENTS.SOMBRAAUG_ICEB
	end,

    getProfileIcon = function( self, sim, abilityOwner )
        return self.profile_icon
    end,
	
	acquireTargets = function( self, targets, game, sim, abilityOwner, unit )
			local x0, y0 = unit:getLocation()
			local maxRange = unit:getTraits().sombra_wireless_range
			if unit:getTraits().wireless_range then
				local maxRange = unit:getTraits().sombra_wireless_range + (unit:getTraits().wireless_range / 2)
			end
			local units = {}
			for _, targetUnit in pairs(sim:getAllUnits()) do
				local x1, y1 = targetUnit:getLocation()
				if x1 and self:isTarget( abilityOwner, unit, targetUnit ) then
					local range = mathutil.dist2d( x0, y0, x1, y1 )
					   if maxRange then
							if range <= maxRange then
								targetUnit = unit:getPlayerOwner():getLastKnownUnit( sim, targetUnit:getID() )
								if targetUnit:isGhost() or sim:canPlayerSeeUnit( unit:getPlayerOwner(), targetUnit ) then
									table.insert( units, targetUnit )
								end
							end
						end
				end
			end

			return targets.unitTarget( game, units, self, abilityOwner, unit )
		end,
		
	isTarget = function( self, abilityOwner, unit, targetUnit )

		if (icewall(targetUnit:getTraits().mainframe_ice) or 0) > 0 and targetUnit:getTraits().mainframe_status ~= "off" then
			return true
		end
		return false
	end,
		
	canUseAbility = function( self, sim, unit, ownerUnit, targetUnitID )

		-- Must have a user owner.
		local userUnit = unit:getUnitOwner()
		if not userUnit then
			return false
		end
		if unit:getTraits().cooldown and unit:getTraits().cooldown > 0 then
			return false, util.sformat(STRINGS.UI.REASON.COOLDOWN,unit:getTraits().cooldown)
		end
		if unit:getTraits().usesCharges and unit:getTraits().charges < 1 then
			return false, util.sformat(STRINGS.UI.REASON.CHARGES)
		end		
        if (unit:getTraits().ammo or 0) <= 0 then
            return false
        end
		
		if userUnit:getAP() < 1 then 
			return false, STRINGS.UI.REASON.ATTACK_USED
		end 
		
		local targetUnit = sim:getUnit( targetUnitID )
		--local ownerUnit = ownerUnit:getUnitOwner()
			if targetUnit then
				local los = false
				if sim:canUnitSeeUnit( ownerUnit, targetUnit ) then
					los = true
				else
					for _, cameraUnit in pairs(sim:getAllUnits()) do 
						if cameraUnit:getTraits().peekID == ownerUnit:getID() and sim:canUnitSeeUnit( cameraUnit, targetUnit ) then
							los = true
						end
					end
				end
				if not los then
					return false, STRINGS.TALON.ABILITIES.EMP.REQ_LOS
				end
				if (unit:getTraits().ammo or 0) < icewall(targetUnit:getTraits().mainframe_ice) then
					return false
				end
			end
		
        return abilityutil.checkRequirements( unit, userUnit)
	end,
		
	executeAbility = function( self, sim, unit, userUnit, target )
		local mainframe = include( "sim/mainframe" )
		local userUnit = unit:getUnitOwner()
		local target = sim:getUnit(target)			
			
		local x0,y0 = userUnit:getLocation()
		local x1,y1 = target:getLocation()
  		local facing = simquery.getDirectionFromDelta( x1 - x0, y1 - y0 )
			simquery.suggestAgentFacing(unit, facing)
		sim:dispatchEvent( simdefs.EV_UNIT_WIRELESS_SCAN, { unitID = unit:getID(), targetUnitID = target, hijack = true } )
			
		local iceCount = unit:getTraits().ammo
        if unit:getTraits().maxAmmo then
            unit:getTraits().ammo = math.max( 0, unit:getTraits().ammo - icewall(target:getTraits().mainframe_ice) )
        end
		target:processEMP( 1 )
		userUnit:useAP( sim )

		--inventory.useItem( sim, userUnit, unit )

		if userUnit:isValid() then
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = userUnit  } )
		end
	end,
}

return sombra_icebreak