
local util = include( "modules/util" )
local array = include( "modules/array" )
local unitdefs = include( "sim/unitdefs" )
local simunit = include( "sim/simunit" )
local simdefs = include( "sim/simdefs" )
local simquery = include( "sim/simquery" )
local simfactory = include( "sim/simfactory" )
local abilityutil = include( "sim/abilities/abilityutil" )


local cdefs = include( "client_defs" )
local inventory = include("sim/inventory")
local speechdefs = include("sim/speechdefs")
local mathutil = include( "modules/mathutil" )

local sombra_tport_translocator = 
	{
		name = STRINGS.ABILITIES.USABLE,

		getName = function( self, sim, unit )
			return STRINGS.ABILITIES.RELOAD
		end,
		
	--	createToolTip = function( self,sim,unit,targetCell)
	--		return abilityutil.formatToolTip(STRINGS.ABILITIES.RELOAD, STRINGS.ABILITIES.RELOAD_DESC, 1 )
	--	end,
	
		onTooltip = function( self, hud, sim, abilityOwner, abilityUser, targetUnitID )
			local tooltip = util.tooltip( hud._screen )
			local section = tooltip:addSection()
			local canUse, reason = abilityUser:canUseAbility( sim, self, abilityOwner, targetUnitID )		
			local targetUnit = sim:getUnit( targetUnitID )
	        section:addLine( STRINGS.TALON.ITEMS.SOMBRA_TRANSLOCATOR )
			section:addAbility( util.sformat( STRINGS.TALON.ITEMS.SOMBRA_TRANSLOCATOR_JUMPTO ),
						util.sformat( STRINGS.TALON.ITEMS.SOMBRA_TRANSLOCATOR_JUMPTO_DESC ), "gui/icons/Talon/icon-action_translocate_jumpto.png" )
			if (targetUnit:getTraits().beaconLife or 0) > 0 then
				--section:addLine( util.sformat( STRINGS.TALON.ITEMS.SOMBRA_TRANSLOCATOR_STATE_1 ) )
				section:addAbility( util.sformat( STRINGS.TALON.ITEMS.SOMBRA_TRANSLOCATOR_SELFDES ),
						util.sformat( STRINGS.TALON.ITEMS.SOMBRA_TRANSLOCATOR_STATE_1 ), "gui/icons/arrow_small.png" )
				else
				section:addAbility( util.sformat( STRINGS.TALON.ITEMS.SOMBRA_TRANSLOCATOR_SELFDES ),
						util.sformat( STRINGS.TALON.ITEMS.SOMBRA_TRANSLOCATOR_STATE_2 ), "gui/icons/arrow_small.png" )
			end
			if reason then
				section:addRequirement( reason )
			end
			return tooltip
		end,

		profile_icon = "gui/icons/Talon/icon-action_translocate.png",		

		--alwaysShow = true,
		proxy = true,
		--ghostable = true,
		
		acquireTargets = function( self, targets, game, sim, unit, userUnit )
			local units = {}
			local x0, y0 = userUnit:getLocation()
			for _, beacon in pairs(sim:getAllUnits()) do
                if beacon:getTraits().isTranslocatorBeacon then
					table.insert( units, beacon )
				end
			end

			return targets.unitTarget( game, units, self, unit, userUnit )
		end,
		
		canUseAbility = function( self, sim, unit, userUnit, targetUnitID )			
			local userUnit = unit:getUnitOwner()
			local targetUnit = sim:getUnit( targetUnitID )
			
		--	if not targetUnit then
        --        return false
        --    end
			if targetUnit then
				local cell = sim:getCell(targetUnit:getLocation())
				if cell.impass > 0 or sim:getQuery().checkDynamicImpass(sim, cell) then
					return false, STRINGS.UI.REASON.BLOCKED
				end
			end
	--		if sim:canPlayerSeeUnit( sim:getNPC(), userUnit ) then       -- From Cloak - Cannot hack when seen
	--					return false, STRINGS.TALON.ABILITIES.EMP.IN_DANGER
	--		end

			return true
		end,

		executeAbility = function ( self, sim, unit, userUnit, targetUnitID )
			--local x0,y0 = userUnit:getLocation()
			--local x1,y1 = unit:getLocation()	
			--local facing = simquery.getDirectionFromDelta(x1-x0,y1-y0)
			local userUnit = unit:getUnitOwner()
			userUnit:setInvisible(false)

				local targetUnit = sim:getUnit( targetUnitID )
				
				if targetUnit then
					local cell = sim:getCell(targetUnit:getLocation())
					sim:dispatchEvent( simdefs.EV_UNIT_APPEARED, { unitID = userUnit:getID(), noSightingFx=true } )
					sim:dispatchEvent( simdefs.EV_TELEPORT, { units={userUnit}, warpOut =true } )
					sim:warpUnit( userUnit, cell )
					sim:dispatchEvent( simdefs.EV_UNIT_APPEARED, { unitID = userUnit:getID(), noSightingFx=true } )
					sim:dispatchEvent( simdefs.EV_TELEPORT, { units={userUnit}, warpOut =false } )
					sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = userUnit } )
					sim:processReactions( userUnit )

					local x0,y0 = targetUnit:getLocation()
					--sim:dispatchEvent( simdefs.EV_FLASH_VIZ, {x = x0, y = y0, units = nil, range = 1} )
					sim:warpUnit(targetUnit, nil)
					sim:despawnUnit( targetUnit )
					local player = sim:getPC()
					player:glimpseUnit(sim, targetUnitID)
					sim:removeTrigger( simdefs.TRG_UNIT_WARP, self)
				end		
								
				--self:getTraits().cooldown = self:getTraits().coolDownUse	

	--		end	
			
		end,
	}
return sombra_tport_translocator