local array = include( "modules/array" )
local util = include( "modules/util" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local abilityutil = include( "sim/abilities/abilityutil" )
local simunit = include( "sim/simunit" )
local simfactory = include( "sim/simfactory" )
local mathutil = include( "modules/mathutil" )


local widow_guntoggle_tooltip = class( abilityutil.hotkey_tooltip )

function widow_guntoggle_tooltip:init( hud, unit, ... )
	abilityutil.hotkey_tooltip.init( self, ... )
	self._game = hud._game
	self._unit = unit
end

function widow_guntoggle_tooltip:activate( screen )
	local widow_aimBonus = WIDOWAIM_BONUS

	abilityutil.hotkey_tooltip.activate( self, screen )
    if self._unit:getTraits().isAutoMode then
   	    self._game.hud:previewAbilityAP( self._unit, -1 )
    else
   	    self._game.hud:previewAbilityAP( self._unit, 1 )
    end
end

function widow_guntoggle_tooltip:deactivate()
	abilityutil.hotkey_tooltip.deactivate( self )
	self._game.hud:previewAbilityAP( self._unit, 0 )
end

local widow_guntoggle = 
	{
		name = STRINGS.TALON.ITEMS.WIDOW_RIFLE_TOOLTIP_TOGGLE,
		createToolTip = function( self,sim,unit,targetCell)
			if unit:getTraits().isAutoMode then
				return abilityutil.formatToolTip( STRINGS.TALON.ITEMS.WIDOW_RIFLE_TOOLTIP_TOGGLE,  STRINGS.TALON.ITEMS.WIDOW_RIFLE_TOOLTIP_TOGGLE_SNIPER_DESC, 1 )
			else
				return abilityutil.formatToolTip( STRINGS.TALON.ITEMS.WIDOW_RIFLE_TOOLTIP_TOGGLE,  STRINGS.TALON.ITEMS.WIDOW_RIFLE_TOOLTIP_TOGGLE_ASSAULT_DESC, 1 )
			end
		end,

		--profile_icon = "gui/items/icon-item_ammo.png",
		profile_icon = "gui/icons/action_icons/Action_icon_Small/icon-item_shoot_small.png",

		getProfileIcon =  function( self, sim, itemUnit )
			if itemUnit:getTraits().isAutoMode then
				return "gui/icons/Talon/icon-action-widow-toggleSnip.png"
			else
				return "gui/icons/Talon/icon-action-widow-toggleKO.png"
			end
		end,
		alwaysShow = true,

		getName = function( self, sim, unit )
			--if unit:getTraits().isAutoMode then
				return STRINGS.TALON.ITEMS.WIDOW_RIFLE_TOGGLEMODE
			--else
				--return STRINGS.TALON.ITEMS.WIDOW_RIFLE_TOOLTIP_TOGGLE_ASSAULT_DESC
			--end
		end,
		
		onSpawnAbility = function( self, sim, unit )
			self.abilityOwner = unit
			sim:addTrigger( simdefs.TRG_END_TURN, self )
		end,
			
		onDespawnAbility = function( self, sim, unit )
			sim:removeTrigger( simdefs.TRG_END_TURN, self )
			self.abilityOwner = nil
		end,

		canUseAbility = function( self, sim, itemUnit, unit )
			if unit:getTraits().mp < 4 and itemUnit:getTraits().isAutoMode == true then
				return false, STRINGS.TALON.ITEMS.WIDOW_RIFLE_AP_REQ
			end
			local userUnit = itemUnit:getUnitOwner()
			
		--	if userUnit:getAP() < 1 then 
		--		return false, STRINGS.UI.REASON.ATTACK_USED
		--	end 
			
	--		if userUnit:getTraits().mp < userUnit:getTraits().mpMax and itemUnit:getTraits().isAutoMode then 
	--			return false, STRINGS.UI.REASON.MUST_TOGGLE_BEFORE_MP
	--		end 
			
	--		if userUnit:getTraits().mp < 1 and not itemUnit:getTraits().isAutoMode then 
	--			return false, STRINGS.UI.REASON.MUST_TOGGLE_BEFORE_MP
	--		end 
			
			return true
		end,
		
		
		onTrigger = function( self, sim, evType, evData )
			local itemUnit = self.abilityOwner
			local player = itemUnit:getPlayerOwner()
			if evType == simdefs.TRG_END_TURN then
			--	if abilityOwner:getTraits().storedCooldown > 0 then
			--		abilityOwner:getTraits().storedCooldown = abilityOwner:getTraits().cooldown - 1
			--	end
				if itemUnit:getTraits().isAutoMode == false then
					itemUnit:getTraits().isAutoMode = true

					itemUnit:getTraits().cooldownMax = nil
					itemUnit:getTraits().storedCooldown = nil
					itemUnit:getTraits().cooldown = nil
					itemUnit:getTraits().baseDamage = itemUnit:getTraits().autoDamage
					itemUnit:getTraits().armorPiercing = itemUnit:getTraits().autoArmorPiercing
					itemUnit:getTraits().canSleep = true
				
					--itemUnit:getTraits().weaponType = "pistol"

					itemUnit:getTraits().storedAmmo = itemUnit:getTraits().ammo
					itemUnit:getTraits().ammo = nil
					itemUnit:getTraits().maxAmmo = nil
					itemUnit:getTraits().charges = itemUnit:getTraits().storedCharges
					itemUnit:getTraits().chargesMax = 2
					itemUnit:getTraits().usesCharges = true
				end
			end
		end,
		
		
		executeAbility = function( self, sim, itemUnit )
		--local weaponUnit = simquery.getEquippedGun( unit )
		--local cell = sim:getCell( unit:getLocation() )
		
		local userUnit = itemUnit:getUnitOwner()
		
			if itemUnit:getTraits().isAutoMode then
                itemUnit:getTraits().isAutoMode = false
				
				itemUnit:getTraits().cooldownMax = nil
				itemUnit:getTraits().storedCooldown = nil
				itemUnit:getTraits().cooldown = nil
				itemUnit:getTraits().baseDamage = itemUnit:getTraits().sniperDamage
				itemUnit:getTraits().armorPiercing = itemUnit:getTraits().sniperArmorPiercing
				itemUnit:getTraits().canSleep = false				
				
				itemUnit:getTraits().ammo = itemUnit:getTraits().storedAmmo
				itemUnit:getTraits().storedAmmo = nil
				itemUnit:getTraits().maxAmmo = 1
				itemUnit:getTraits().storedCharges = itemUnit:getTraits().charges
				itemUnit:getTraits().charges = nil
				itemUnit:getTraits().chargesMax = nil
				itemUnit:getTraits().usesCharges = nil		
				
				--itemUnit:getTraits().weaponType = "rifle"
				--itemUnit:getTraits().modTrait = {{"mpMax",itemUnit:getTraits().sniperMpMod}}
				userUnit:getTraits().mp = userUnit:getTraits().mp - 4
				
				
				--userUnit:getTraits().mp = 1
				
				--sim:addTrigger( simdefs.TRG_END_TURN, self )   --------- add trigger end turn

				if userUnit then
					local x0, y0 = userUnit:getLocation()
					--sim:emitSound( { path = "SpySociety/Actions/recharge_item", range = simdefs.SOUND_RANGE_0 }, x0, y0, userUnit )
					--sim:emitSound( simdefs.SOUND_PRIME_EMP, x0, x0, userUnit)
					--sim:dispatchEvent( simdefs.EV_UNIT_PICKUP, { unitID = userUnit:getID() } )
					sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=STRINGS.TALON.ITEMS.WIDOW_RIFLE_SNIPER, unit = userUnit, color= cdefs.MOVECLR_DEFAULT })
				end
				--sim:dispatchEvent( simdefs.EV_UNIT_DEPLOY, {unit=itemUnit, x=cell.x,y=cell.y})

            else
                itemUnit:getTraits().isAutoMode = true

				itemUnit:getTraits().cooldownMax = nil
				itemUnit:getTraits().storedCooldown = nil
				itemUnit:getTraits().cooldown = nil
				itemUnit:getTraits().baseDamage = itemUnit:getTraits().autoDamage
				itemUnit:getTraits().armorPiercing = itemUnit:getTraits().autoArmorPiercing
				itemUnit:getTraits().canSleep = true
			
				--itemUnit:getTraits().weaponType = "pistol"

				itemUnit:getTraits().storedAmmo = itemUnit:getTraits().ammo
				itemUnit:getTraits().ammo = nil
				itemUnit:getTraits().maxAmmo = nil
				itemUnit:getTraits().charges = itemUnit:getTraits().storedCharges
				itemUnit:getTraits().chargesMax = itemUnit:getTraits().autoChargesMax
				itemUnit:getTraits().usesCharges = true
				
				--itemUnit:getTraits().modTrait = {{"mpMax",itemUnit:getTraits().autoMpMod}}
				userUnit:getTraits().mp = userUnit:getTraits().mp + 4
				

				--userUnit:getTraits().mp = userUnit:getTraits().mpMax
				
    			if userUnit then
					local x0, y0 = userUnit:getLocation()
					--sim:emitSound( { path = "SpySociety/Actions/recharge_item", range = simdefs.SOUND_RANGE_0 }, x0, y0, userUnit )
					--sim:emitSound( simdefs.SOUND_PRIME_EMP, x0, y0, userUnit)
					--sim:dispatchEvent( simdefs.EV_UNIT_PICKUP, { unitID = userUnit:getID() } )
					sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=STRINGS.TALON.ITEMS.WIDOW_RIFLE_ASSAULT, unit = userUnit, color= cdefs.MOVECLR_DEFAULT })
				end

			end

		end,

	}

return widow_guntoggle