local array = include( "modules/array" )
local util = include( "modules/util" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local inventory = include("sim/inventory")
local abilityutil = include( "sim/abilities/abilityutil" )
local mathutil = include( "modules/mathutil" )

local widow_rappel =
{
	name = STRINGS.TALON.ITEMS.AUGMENTS.SOMBRAAUG_ICEB,
	profile_icon = "gui/icons/Talon/icon-item_sombra_hijack_small.png", 
    proxy = true,
	usesAction = true,
	alwaysShow = true,

--	createToolTip = function( self,sim, abilityOwner, abilityUser, targetID )
--		local targetUnit = sim:getUnit( targetID )
--		local headerTxt = util.sformat( STRINGS.TALON.ITEMS.AUGMENTS.SOMBRAAUG_TARGNAME, util.toupper( targetUnit:getName() ) )
--		local bodyTxt = util.sformat( STRINGS.TALON.ITEMS.AUGMENTS.SOMBRAAUG_WILLDO, targetUnit:getTraits().mainframe_ice, abilityOwner:getTraits().ammo )
--		return abilityutil.formatToolTip( headerTxt, bodyTxt, simdefs.DEFAULT_COST )
--	end,
	createToolTip = function( self,sim,unit,targetCell)
		return abilityutil.formatToolTip( STRINGS.ABILITIES.THROW,  STRINGS.ABILITIES.THROW_DESC, 1 )
	end,
		
	getName = function( self, sim, unit )
		return STRINGS.TALON.ITEMS.AUGMENTS.SOMBRAAUG_ICEB
	end,

    getProfileIcon = function( self, sim, abilityOwner )
        return self.profile_icon
    end,
	
	acquireTargets = function( self, targets, game, sim, abilityOwner, unit )
			local x0, y0 = unit:getLocation()
			local maxRange = 6
			local units = {}
			for _, targetUnit in pairs(sim:getAllUnits()) do
				local x1, y1 = targetUnit:getLocation()
				if x1 and self:isTarget( abilityOwner, unit, targetUnit ) then
					local range = mathutil.dist2d( x0, y0, x1, y1 )
					   if maxRange then
							if range <= maxRange then
								targetUnit = unit:getPlayerOwner():getLastKnownUnit( sim, targetUnit:getID() )
								if targetUnit:isGhost() or sim:canPlayerSeeUnit( unit:getPlayerOwner(), targetUnit ) then
									table.insert( units, targetUnit )
								end
							end
						end
				end
			end

		return targets.unitTarget( game, units, self, abilityOwner, unit )
	end,
		
	isTarget = function( self, abilityOwner, unit, targetUnit )

		if targetUnit:getTraits().mainframe_item or targetUnit:getTraits().mainframe_console then
			return true
		end
		return false
	end,
		
	canUseAbility = function( self, sim, unit, ownerUnit, targetUnitID )

		-- Must have a user owner.
		local userUnit = unit:getUnitOwner()
		if not userUnit then
			return false
		end
		if unit:getTraits().cooldown and unit:getTraits().cooldown > 0 then
			return false, util.sformat(STRINGS.UI.REASON.COOLDOWN,unit:getTraits().cooldown)
		end
		if unit:getTraits().usesCharges and unit:getTraits().charges < 1 then
			return false, util.sformat(STRINGS.UI.REASON.CHARGES)
		end		
        if (unit:getTraits().ammo or 0) <= 0 then
            return false
        end
		
		if userUnit:getAP() < 1 then 
			return false, STRINGS.UI.REASON.ATTACK_USED
		end 
		
	--	if userUnit:getTraits().movingBody then
    --            return false, STRINGS.UI.REASON.DROP_BODY_TO_USE
    --    end
		
		local targetUnit = sim:getUnit( targetUnitID )
		--local ownerUnit = abilityOwner:getUnitOwner()
			if targetUnit then
				if not sim:canUnitSeeUnit( userUnit, targetUnit ) --[==[and curRange > 1 ]==] then
					return false, STRINGS.TALON.ABILITIES.EMP.REQ_LOS   -- requires LOS or proximity
				end
			end
		
        return abilityutil.checkRequirements( unit, userUnit)
	end,
		
	executeAbility = function( self, sim, unit, userUnit, target )
		local mainframe = include( "sim/mainframe" )
		local userUnit = unit:getUnitOwner()
		local target = sim:getUnit(target)			
			
		local cell = sim:getCell(target:getLocation())
		local x0,y0 = userUnit:getLocation()
		local x1,y1 = target:getLocation()
  		local facing = simquery.getDirectionFromDelta( x1 - x0, y1 - y0 )
		
		local cells = simquery.fillCircle( sim, x1, y1, 1.5, 0)
		local closestCell, closestDist
		for i, testcell in ipairs(cells) do
		--	if not testcell.units then
				local x2,y2 = testcell()
				local dist = mathutil.dist2d( x2, y2, x1, y1 )
				if not closestDist or dist < closestDist then
					closestDist = dist
					closestItem = testcell
				end
		--	end
		end
		
		
		
		simquery.suggestAgentFacing(unit, facing)
		
		userUnit:getTraits().throwing = true
		
		local facing = simquery.getDirectionFromDelta(x1-x0, y1-y0)
		simquery.suggestAgentFacing(userUnit, facing)
		
		if userUnit:isValid() and not userUnit:getTraits().interrupted then
			sim:dispatchEvent( simdefs.EV_UNIT_THROW, { unit = userUnit, x1=x1, y1=y1, facing=facing } )
		
			sim:warpUnit( userUnit, closestCell )
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = userUnit  } )
		end
		
		sim:dispatchEvent( simdefs.EV_UNIT_STOP_THROW, { unitID = userUnit:getID(), x1=x1, y1=y1, facing=facing } )
		
		inventory.useItem( sim, userUnit, unit )

		if userUnit:isValid() then
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = userUnit  } )
		end
	end,
}

return widow_rappel